@@ -2,8 +2,8 @@ from typing import Callable, List, Optional
 
 from google.protobuf.struct_pb2 import NULL_VALUE
 
+from preemo.gen.endpoints.batch_execute_function_pb2 import BatchExecuteFunctionRequest
 from preemo.gen.endpoints.check_function_pb2 import CheckFunctionRequest
-from preemo.gen.endpoints.execute_function_pb2 import ExecuteFunctionRequest
 from preemo.gen.endpoints.register_function_pb2 import RegisterFunctionRequest
 from preemo.gen.models.registered_function_pb2 import RegisteredFunction
 from preemo.gen.models.value_pb2 import Value
@@ -13,6 +13,20 @@ from preemo.worker._messaging_client import IMessagingClient
 from preemo.worker._types import assert_never
 
 
+class Result:
+    # TODO(adrian@preemo.io, 04/20/2023): need to sort out how this class
+    # can be used when error handling, status checking, etc
+    # Perhaps futures should be used instead
+    def __init__(
+        self, *, artifact_id: ArtifactId, artifact_manager: IArtifactManager
+    ) -> None:
+        self._artifact_id = artifact_id
+        self._artifact_manager = artifact_manager
+
+    def get(self) -> bytes:
+        return self._artifact_manager.get_artifact(self._artifact_id)
+
+
 class Function:
     def __init__(
         self,
@@ -39,15 +53,15 @@ class Function:
             )
         )
 
-    def __call__(self, params: Optional[str] = None) -> Optional[str]:
+    def __call__(self, params: Optional[bytes] = None) -> Optional[Result]:
         if params is None:
             function_parameter = Value(null_value=NULL_VALUE)
         else:
             artifact_id = self._artifact_manager.create_artifact(params)
             function_parameter = Value(artifact_id=artifact_id.value)
 
-        response = self._messaging_client.execute_function(
-            ExecuteFunctionRequest(
+        response = self._messaging_client.batch_execute_function(
+            BatchExecuteFunctionRequest(
                 function_to_execute=RegisteredFunction(
                     name=self.name, namespace=self.namespace
                 ),
@@ -65,8 +79,9 @@ class Function:
             return None
 
         if kind == "artifact_id":
-            return self._artifact_manager.get_artifact(
-                ArtifactId(value=function_result.artifact_id)
+            return Result(
+                artifact_id=ArtifactId(value=function_result.artifact_id),
+                artifact_manager=self._artifact_manager,
             )
 
         assert_never(kind)
@@ -74,12 +89,15 @@ class Function:
 
 class WorkerClient:
     def __init__(
-        self, *, artifact_manager: IArtifactManager, messaging_client: IMessagingClient
+        self,
+        *,
+        artifact_manager: IArtifactManager,
+        function_registry: FunctionRegistry,
+        messaging_client: IMessagingClient,
     ) -> None:
         self._artifact_manager = artifact_manager
         self._messaging_client = messaging_client
-
-        self._function_registry = FunctionRegistry()
+        self._function_registry = function_registry
 
     def get_function(self, name: str, *, namespace: Optional[str] = None) -> Function:
         return Function(
@@ -89,13 +107,13 @@ class WorkerClient:
             namespace=namespace,
         )
 
-    def parallelize(
+    def parallel(
         self,
         function: Function,
         *,
-        params: Optional[List[str]] = None,
+        params: Optional[List[bytes]] = None,
         count: Optional[int] = None,
-    ) -> List[Optional[str]]:
+    ) -> List[Optional[Result]]:
         # TODO(adrian@preemo.io, 03/20/2023): should take an optional config argument includes stuff like max batch size
 
         if params is None:
@@ -121,8 +139,8 @@ class WorkerClient:
                 for i, artifact_id in enumerate(artifact_ids)
             }
 
-        response = self._messaging_client.execute_function(
-            ExecuteFunctionRequest(
+        response = self._messaging_client.batch_execute_function(
+            BatchExecuteFunctionRequest(
                 function_to_execute=RegisteredFunction(
                     name=function.name, namespace=function.namespace
                 ),
@@ -130,8 +148,7 @@ class WorkerClient:
             )
         )
 
-        # TODO(adrian@preemo.io, 03/20/2023): should download results in parallel or return a handle that allows the user to download result
-        results: List[Optional[str]] = []
+        results: List[Optional[Result]] = []
         for _, function_result in sorted(
             response.results_by_index.items(), key=lambda x: x[0]
         ):
@@ -143,8 +160,9 @@ class WorkerClient:
                 results.append(None)
             elif kind == "artifact_id":
                 results.append(
-                    self._artifact_manager.get_artifact(
-                        ArtifactId(value=function_result.artifact_id)
+                    Result(
+                        artifact_id=ArtifactId(value=function_result.artifact_id),
+                        artifact_manager=self._artifact_manager,
                     )
                 )
             else:
